@@ -164,7 +164,7 @@ static const char _datB[]="\x01\x02"
 #define OFFSET 0x88                  /* Allow replacing sprite ranges with offsets */
 #define W(cnt) cnt & 0xFF, cnt >> 8  /* Construct word count */
 static const unsigned char _dat5[]={
-NDF_HEADER(0x04, 12),
+NDF_HEADER(0x04, 15),
 /*04*/ OFFSET,          OPTIONS(3), 0x30, 0x70, 0xF0,
 /*05*/ OFFSET,          OPTIONS(1), 0x30,
 /*06*/ OFFSET,          OPTIONS(2), 0x4A, 0x5A,
@@ -182,7 +182,7 @@ NDF_HEADER(0x04, 12),
 /*12*/ OFFSET,          OPTIONS(1), 0x08,
 /*13*/ OFFSET,          OPTIONS(1), 0x37,
 /*14*/ OFFSET,          OPTIONS(1), 0x24,
-/*15*/ OFFSET,          OPTIONS(1), 0xA2,
+/*15*/ OFFSET,          OPTIONS(1), 0xAE,
 /*16*/ OFFSET,          OPTIONS(1), 0x09,
 /*17*/ OFFSET,          OPTIONS(1), 0x10,
 00,
@@ -238,8 +238,8 @@ static const char _datTextIDs[]="\x04\x09"
 #define GROUNDVEHICLE MASK(TRAIN) | MASK(ROADVEH)
 #define VEHICLE MASK(TRAIN) | MASK(ROADVEH) | MASK(SHIP) | MASK(AIRCRAFT)
 static const unsigned char _datcallbacks[]={
-NDF_HEADER(0x05, 21),
-/* Count: */ W(0x15F),
+NDF_HEADER(0x05, 23),
+/* Count: */ W(0x160),
 /* 00*/ NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE,
 /* 10*/ GROUNDVEHICLE | MASK(SHIP), GROUNDVEHICLE, VEHICLE, STATION, STATION, VEHICLE, GROUNDVEHICLE, HOUSE,
 /* 18*/ MASK(STATION) | VEHICLE, VEHICLE, HOUSE, HOUSE, HOUSE, TRAIN, HOUSE, HOUSE,
@@ -266,7 +266,7 @@ NDF_HEADER(0x05, 21),
 /*140*/ STATION, STATION, STATION, HOUSE, SOUND, CARGO, SIGNAL, CANAL,
 /*148*/ HOUSE, STATION, INDUSTRY, INDUSTRY, INDUSTRY, HOUSE, HOUSE, HOUSE,
 /*150*/ AIRTILE, NONE, AIRTILE, AIRTILE, AIRTILE, AIRPORT, AIRPORT, OBJECT,
-/*158*/ OBJECT, OBJECT, OBJECT, OBJECT, OBJECT, OBJECT, VEHICLE,
+/*158*/ OBJECT, OBJECT, OBJECT, OBJECT, OBJECT, OBJECT, VEHICLE, INDUSTRY,
 NDF_END
 };
 #undef W
@@ -298,14 +298,14 @@ NDF_END
  *
  * Names of languages by ID (empty if not defined), each terminated by newline
  */
-static const char _datlangs[]="\x00\x06"
+static const char _datlangs[]="\x00\x07"
 //      x0/x4/x8/xC               x1/x5/x9/xD               x2/x6/xA/xE               x3/x7/xB/xF
 /*0x*/  "American\n"              "English\n"               "German\n"                "French\n"
 /*0x*/  "Spanish\n"               "Esperanto\n"             "Ido\n"                   "Russian\n"
 /*0x*/  "Irish\n"                 "Maltese\n"               "Tamil\n"                 "Chuvash\n"
 /*0x*/  "Chinese (Traditional)\n" "Serbian\n"               "Norwegian (Nynorsk)\n"   "Welsh\n"
 //      x0/x4/x8/xC               x1/x5/x9/xD               x2/x6/xA/xE               x3/x7/xB/xF
-/*1x*/  "Belarusian\n"            "Marathi\n"               "Faroese\n"               "\n"
+/*1x*/  "Belarusian\n"            "Marathi\n"               "Faroese\n"               "Scottish Gaelic\n"
 /*1x*/  "Arabic (Egypt)\n"        "Czech\n"                 "Slovak\n"                "\n"
 /*1x*/  "Bulgarian\n"             "\n"                      "\n"                      "Afrikaans\n"
 /*1x*/  "\n"                      "\n"                      "Greek\n"                 "Dutch\n"
@@ -474,10 +474,24 @@ NDF_END
 
 /* Raw byte to appear in the nfo */
 #define RAW(b) 'l', b
+#define RAWBYTES(len, ...) 'm', len, __VA_ARGS__
+
+/* Test for a specific byte value
+ * 'offset' is the position of the byte to test within the current SUBDATA.
+ * 'mask' is a AND bitmask to apply before testing.
+ * 'value' is the value to test for.
+ * 'data' is the stuff to process when the test "(byte[offset] & mask) == value" succeeds. */
+#define IF(lhs, rhs, data) 'e', lhs, rhs, data
+#define IFNOT(lhs, rhs, data) 'n', lhs, rhs, data
+#define ASSERT 'a'
+#define SKIPNEXT 's'
+
+/* Bitwise AND */
+#define AND(lhs, rhs) '&', lhs, rhs
 
 /* Repeat 'data' 'times' times, where 'times' is known before reading the data.
  * 'times' may be a fixed number, or a reference to an earlier BYTE/EXTBYTE/WORD/DWORD within the current SUBDATA.
- * You can also multiple multiple values using MUL() */
+ * You can also multiply multiple values using MUL() */
 #define REPEAT_N(data, times) 'r', data, times
 #define MUL(a, b) 'x', a, b
 #define DATAFROM(offset) offset | 0x80           /* Uses the value read with the token at position 'offset' within the current SUBDATA */
@@ -486,7 +500,7 @@ NDF_END
 #define REPEAT_UNTIL(data, length, ...) '*', data, length, __VA_ARGS__
 
 static const unsigned char _dat0[]={
-NDF_HEADER(0x0D, 21),
+NDF_HEADER(0x0E, 0),
 /*Maximum feature:*/ 0x11,
 // Feature 00:
 /*00*/ WORD | DATE, INVALID, BYTE, BYTE | DECIMAL, BYTE | DECIMAL, BYTE, BYTE, BYTE | DECIMAL,
@@ -539,15 +553,57 @@ BYTE, REPEAT_N(BYTE, DATAFROM(0)), END,
 /*00*/ INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID,
 /*08*/ DWORD | QUOTED, SUBDATA, BYTE, BYTE, BYTE, BYTE, SUBDATA, BYTE,
 /*10*/ WORD | DECIMAL, BYTE, DWORD | HEX, BYTE, BYTE, BYTE, WORD | HEX, BYTE | DECIMAL,
-/*18*/ WORD,
+/*18*/ WORD, INVALID, SUBDATA,
 END,
 // Subdata - prop 09:
 EXTBYTE, REPEAT_N(SUBDATA, DATAFROM(0)), END,
-	RAW(ZERO), RAW(ZERO), RAW(ZERO), RAW(ZERO), '|', DWORD | HEX, REPEAT_UNTIL(SUBDATA, 1, 0x80), END,
-		BYTE, BYTE, BYTE, BYTE, BYTE, BYTE, DWORD | HEX, END,
+	LINEBREAK, RAW(ZERO), RAW(ZERO), RAW(ZERO), RAW(ZERO), '|', LINEBREAK, DWORD | HEX, LINEBREAK, REPEAT_UNTIL(SUBDATA, 1, 0x80), END,
+		BYTE, BYTE, BYTE, BYTE, BYTE, BYTE, DWORD | HEX, LINEBREAK, END,
 // Subdata - prop 0E:
 REPEAT_UNTIL(SUBDATA, 2, ZERO, ZERO), END,
 	BYTE, BYTE, REPEAT_N(BYTE, MUL(DATAFROM(0), DATAFROM(1))), LINEBREAK, END,
+// Subdata - prop 20:
+EXTBYTE, REPEAT_N(SUBDATA, DATAFROM(0)), END,
+	// spritelayout without flags
+	BYTE, IFNOT(AND(DATAFROM(0), RAW(0x40)), ZERO, SKIPNEXT), DWORD, LINEBREAK, REPEAT_N(SUBDATA, AND(DATAFROM(0), RAW(0x3F))),
+	// spritelayout with flags
+	'|', BYTE, LINEBREAK, SUBDATA, REPEAT_N(SUBDATA, AND(DATAFROM(0), RAW(0x3F))), END,
+		// building sprite without flags
+		DWORD, BYTE | DECIMAL, BYTE | DECIMAL, RAW(0x80), LINEBREAK,
+		'|', DWORD, BYTE | DECIMAL, BYTE | DECIMAL, BYTE | DECIMAL, BYTE | DECIMAL, BYTE | DECIMAL, BYTE | DECIMAL, LINEBREAK, END,
+		// groundsprite with flags
+		DWORD, WORD | HEX,
+		IFNOT(AND(DATAFROM(1), RAWBYTES(2, 0x30, 0xFF)), ZERO, ASSERT), // assert on unknown flags
+		IFNOT(AND(DATAFROM(1), RAW(0x01)), ZERO, BYTE), // skip sprite
+		IFNOT(AND(DATAFROM(1), RAW(0x02)), ZERO, BYTE), // sprite offset
+		IFNOT(AND(DATAFROM(1), RAW(0x04)), ZERO, BYTE), // palette offset
+		IFNOT(AND(DATAFROM(1), RAW(0x40)), ZERO, BYTE), // sprite var10
+		IFNOT(AND(DATAFROM(1), RAW(0x80)), ZERO, BYTE), // palette var10
+		LINEBREAK,
+		END,
+		// buildingsprite with flags
+		DWORD, WORD | HEX, BYTE | DECIMAL, BYTE | DECIMAL, RAW(0x80),
+		IFNOT(AND(DATAFROM(1), RAWBYTES(2, ZERO, 0xFF)), ZERO, ASSERT), // assert on unknown flags
+		IFNOT(AND(DATAFROM(1), RAW(0x01)), ZERO, BYTE), // skip sprite
+		IFNOT(AND(DATAFROM(1), RAW(0x02)), ZERO, BYTE), // sprite offset
+		IFNOT(AND(DATAFROM(1), RAW(0x04)), ZERO, BYTE), // palette offset
+		IFNOT(AND(DATAFROM(1), RAW(0x10)), ZERO, BYTE), // X offset
+		IFNOT(AND(DATAFROM(1), RAW(0x20)), ZERO, BYTE), // Y offset
+		IFNOT(AND(DATAFROM(1), RAW(0x40)), ZERO, BYTE), // sprite var10
+		IFNOT(AND(DATAFROM(1), RAW(0x80)), ZERO, BYTE), // palette var10
+		LINEBREAK,
+		'|', DWORD, WORD | HEX, BYTE | DECIMAL, BYTE | DECIMAL, BYTE | DECIMAL, BYTE | DECIMAL, BYTE | DECIMAL, BYTE | DECIMAL,
+		IFNOT(AND(DATAFROM(1), RAWBYTES(2, ZERO, 0xFF)), ZERO, ASSERT), // assert on unknown flags
+		IFNOT(AND(DATAFROM(1), RAW(0x01)), ZERO, BYTE), // skip sprite
+		IFNOT(AND(DATAFROM(1), RAW(0x02)), ZERO, BYTE), // sprite offset
+		IFNOT(AND(DATAFROM(1), RAW(0x04)), ZERO, BYTE), // palette offset
+		IFNOT(AND(DATAFROM(1), RAW(0x10)), ZERO, SUBDATA), // X/Y offset
+		IFNOT(AND(DATAFROM(1), RAW(0x20)), ZERO, BYTE), // Z offset
+		IFNOT(AND(DATAFROM(1), RAW(0x40)), ZERO, BYTE), // sprite var10
+		IFNOT(AND(DATAFROM(1), RAW(0x80)), ZERO, BYTE), // palette var10
+		LINEBREAK,
+		END,
+			BYTE, BYTE, END, // X/Y offset
 
 // Feature 05:
 /*00*/ INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID,
@@ -581,7 +637,7 @@ BYTE, REPEAT_N(BYTE, DATAFROM(0)), END,
 //		The max ID that can be set
 // (0f8.dat was merged into here to prevent it from getting out of sync.)
 /*00*/ INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID,
-/*08*/ BYTE,          0x41, 0x41, DWORD | QUOTED, 0x00, 0xFC, TEXTID,         0x12, 0x12, DWORD | DECIMAL, 0x12, 0x12,
+/*08*/ BYTE,          0x46, 0x46, DWORD | QUOTED, 0x00, 0xFC, TEXTID,         0x12, 0x12, DWORD | DECIMAL, 0x12, 0x12,
 /*0C*/ WORD | QUOTED, 0x12, 0x12, DWORD | QUOTED, 0x12, 0x12, DWORD | QUOTED, 0x12, 0x12, WORD  | DECIMAL, 0x12, 0x12,
 /*10*/ SUBDATA,       0x00, 0x00, SUBDATA,        0x00, 0xFF, DWORD | QUOTED, 0x00, 0xFC, SUBDATA,         0x7E, 0x7E,
 /*14*/ SUBDATA,       0x7E, 0x7E, BYTE,           0x7E, 0x7E,
@@ -702,6 +758,12 @@ NDF_END
 #undef LINEBREAK
 #undef SUBDATA
 #undef RAW
+#undef RAWBYTES
+#undef IF
+#undef IFNOT
+#undef ASSERT
+#undef SKIPNEXT
+#undef AND
 #undef REPEAT_N
 #undef MUL
 #undef DATAFROM
@@ -720,13 +782,13 @@ NDF_END
 #define LASTVAR80(var) (var == 0xFF ? 0xFF : var + 1), (var == 0xFF ? 0xF0 : 0x80)
 #define NOVAR80 LASTVAR80(0x7F)
 static const unsigned char _dat2v[]={
-NDF_HEADER(0x0D, 26),
+NDF_HEADER(0x0D, 28),
 /*Maximum feature:*/ 0x11,
 /*Maximum operator:*/ 0x16,
 // Global variables:
-WORD(0x00), BYTE(0x01), BYTE(0x02), BYTE(0x03), WORD(0x09), WORD(0x0A), WORD(0x0C),
-DWORD(0x10), BYTE(0x12), DWORD(0x18), VAR(0x1A, 4 | ALLOW0MASK), BYTE(0x1B), DWORD(0x1C), BYTE(0x1D),
-BYTE(0x20), DWORD(0x23), DWORD(0x24),
+WORD(0x00), BYTE(0x01), BYTE(0x02), BYTE(0x03), BYTE(0x06), WORD(0x09), WORD(0x0A), DWORD(0x0B), WORD(0x0C), BYTE(0x0D), BYTE(0x0E), VAR(0x0F, 3),
+DWORD(0x10), /* var 11 in feature 04 */ BYTE(0x12), WORD(0x13), WORD(0x14), WORD(0x15), WORD(0x16), DWORD(0x18), VAR(0x1A, 4 | ALLOW0MASK), BYTE(0x1B), DWORD(0x1C), BYTE(0x1D), DWORD(0x1E),
+BYTE(0x20), DWORD(0x21), DWORD(0x22), DWORD(0x23), DWORD(0x24),
 DWORD6x(0x7D, 0xFF), VAR6x(0x7E, 2 | ALLOW0MASK, 0xFF), DWORD6x(0x7F, 0x7F),
 // Things like 25, 5F and 7C as feature specific, so there are put with features
 LASTVAR80(0xFF),
@@ -734,24 +796,26 @@ LASTVAR80(0xFF),
 /*Related:*/ 0x00,
 DWORD(0x25),
 VAR(0x40, 3), VAR(0x41, 3), DWORD(0x42), DWORD(0x43), VAR(0x45, 3), DWORD(0x46), DWORD(0x47),
-BYTE(0x48), DWORD(0x49), DWORD(0x4A), DWORD(0x4B),
+BYTE(0x48), DWORD(0x49), DWORD(0x4A), DWORD(0x4B), DWORD(0x4C),
 WORD(0x5F),
-BYTE6x(0x60, 0x73),
+BYTE6x(0x60, 0x73), DWORD6x(0x61, 0xFF), DWORD6x(0x62, 0xFF),
 LASTVAR80(0xFF),
 // Feature 01:
 /*Related:*/ 0x01,
 VAR(0x40, 3), VAR(0x41, 3), VAR(0x42, 3), DWORD(0x43), VAR(0x45, 3), DWORD(0x46), DWORD(0x47),
-BYTE(0x48), DWORD(0x49),
+BYTE(0x48), DWORD(0x49), DWORD(0x4B), DWORD(0x4C),
 WORD(0x5F),
+DWORD6x(0x61, 0xFF), DWORD6x(0x62, 0xFF),
 LASTVAR80(0xFF),
 // Feature 02:
 /*Related:*/ 0x02,
-BYTE(0x42), DWORD(0x43), DWORD(0x46), DWORD(0x47), BYTE(0x48), DWORD(0x49),
+BYTE(0x42), DWORD(0x43), DWORD(0x46), DWORD(0x47), BYTE(0x48), DWORD(0x49), DWORD(0x4B), DWORD(0x4C),
 WORD(0x5F),
 LASTVAR80(0xFF),
 // Feature 03:
 /*Related:*/ 0x03,
 VAR(0x40, 3), BYTE(0x42), DWORD(0x43), WORD(0x44), DWORD(0x46), DWORD(0x47), BYTE(0x48), DWORD(0x49),
+DWORD(0x4B), DWORD(0x4C),
 WORD(0x5F),
 LASTVAR80(0xFF),
 // Feature 04:
@@ -844,9 +908,9 @@ NDF_END
 
 #define W(cnt) cnt & 0xFF, cnt >> 8  /* Construct word count */
 static const unsigned char _datD[]={
-NDF_HEADER(0x14, 6),
+NDF_HEADER(0x14, 7),
 /*Maximum feature:*/ 0x11,
-/*Max patch variable:*/ 0x14,
+/*Max patch variable:*/ 0x16,
 /*Max operator:*/ 0x0C,
 // GRM count:
 /*00*/W(0x74),
