@@ -785,9 +785,25 @@ uint PseudoSprite::ReadValue(istream& in, width w) {
 		in.ignore()>>setbase(16)>>ret>>setbase(10);
 		return ret;
 	}
-	/*if (in.peek() == '(') {		// Read any RPN value
-		//TODO: Magic goes here
-	}*/
+	if (in.peek() == '(') {		// Read any RPN value
+		if(RPNOFF){
+			IssueMessage(0, INVALID_EXTENSION);
+			return 0;
+		}
+		int val, err = 0, s = in.tellg();
+		val = DoCalc(in.ignore(),err);
+		if (err>0)
+			return 0;
+
+		// Replace the original RPN with value
+		int e = in.tellg(),
+			p = orig.find(((istringstream&)in).str().substr(s, e - s));
+		orig.erase(p, e - s);
+		ostringstream Val;
+		Val << val;
+		orig.insert(p, Val.str());
+		return val;
+	}
 
 	// Read any other value
 	string str;
