@@ -30,7 +30,7 @@ static int decodetile(U8 *buffer, int sx, int sy, CommonPixel *imgbuffer, long t
 	bool long_chunk  = grfcontversion == 2 && sx > 256;
 	buffer += SpriteInfo::Size(grfcontversion);
 
-	if (tilesize <= sy * (long_format ? 4 : 2) + SpriteInfo::Size(grfcontversion)) return -1;
+	if (grfcontversion == 2 && tilesize <= sy * (long_format ? 4 : 2)) return -1;
 
 	for (int y=0; y<sy; y++) {
 		long offset;
@@ -45,13 +45,13 @@ static int decodetile(U8 *buffer, int sx, int sy, CommonPixel *imgbuffer, long t
 		long x, islast, chunkstart=0, len, ofs;
 		do {
 			if (long_chunk) {
-				if (offset + 3 >= tilesize) return -1;
+				if (grfcontversion == 2 && offset + 3 >= tilesize) return -1;
 				islast = buffer[offset + 1] & 0x80;
 				len = (buffer[offset + 1] & 0x7f) << 8 | buffer[offset + 0];
 				ofs = buffer[offset + 3] << 8 | buffer[offset + 2];
 				offset += 4;
 			} else {
-				if (offset + 1 >= tilesize) return -1;
+				if (grfcontversion == 2 && offset + 1 >= tilesize) return -1;
 				islast = buffer[offset] & 0x80;
 				len = buffer[offset++] & 0x7f;
 				ofs = buffer[offset++];
@@ -66,7 +66,7 @@ static int decodetile(U8 *buffer, int sx, int sy, CommonPixel *imgbuffer, long t
 
 			// then copy the number of actual bytes
 			const U8 *obuffer = buffer + offset;
-			if (offset + len > tilesize) return -1;
+			if (grfcontversion == 2 && offset + len > tilesize) return -1;
 			for (x=0; x<len; x++) {
 				obuffer = imgbuffer->Decode(obuffer, has_mask, rgba);
 				imgbuffer++;
@@ -200,7 +200,7 @@ void SpriteInfo::writetobuffer(U8 *buffer, int grfcontversion)
 	buffer[i++] = this->yrel >> 8;
 }
 
-void SpriteInfo::readfromfile(const char *action, int grfcontversion, FILE *grf)
+void SpriteInfo::readfromfile(const char *action, int grfcontversion, FILE *grf, int spriteno)
 {
 	int i = 0;
 	if (grfcontversion == 2) {
@@ -211,7 +211,10 @@ void SpriteInfo::readfromfile(const char *action, int grfcontversion, FILE *grf)
 			case 0x03: this->depth=DEPTH_32BPP; break;
 			case 0x04: this->depth=DEPTH_8BPP; break;
 			case 0x07: this->depth=DEPTH_MASK; break;
-			default: this->depth=DEPTH_8BPP; break;
+			default: {
+				printf("\nUnknown colour depth %02x for sprite %d. GRFCodec currently only supports M, RGBA and RGBAM formats.\n", data&0x7, spriteno);
+				exit(2);
+			}
 		}
 		this->zoom = fgetc(grf);
 		this->ydim = readword(action, grf);
@@ -289,17 +292,23 @@ int decodesprite(FILE *grf, spritestorage *imgpal, spritestorage *imgrgba, sprit
 		}
 
 		if (inf == 0xff) {
+			if (returnpos != 0) size--;
+
 			imgpal->setsize(1, 0);
 			outbuffer = (U8*) malloc(size);
 			//outbuffer[0] = 0xff;
 			cfread(action, outbuffer, 1, size, grf);
 			writer->adddata(size, outbuffer/*+1*/);
 			imgpal->spritedone();
-			return 1;
+			result = 1;
+
+			if (returnpos == 0) return result;
+			*dataoffset = startpos + size + 1;
+			break;
 		}
 
 		fseek(grf, startpos, SEEK_SET);
-		info.readfromfile(action, grfcontversion, grf);
+		info.readfromfile(action, grfcontversion, grf, spriteno);
 
 		if(info.zoom >= ZOOM_LEVELS){
 			printf("\nUnknown zoom level %02x for sprite %d\n", info.zoom, spriteno);
@@ -373,7 +382,8 @@ int decodesprite(FILE *grf, spritestorage *imgpal, spritestorage *imgrgba, sprit
 				exit(2);
 			}
 		} else {
-			if (grfcontversion == 2 && datasize != (unsigned long)(sx * sy) + SpriteInfo::Size(grfcontversion)) {
+			U8 bytes_per_pixel=(has_mask?1:0)+(rgba?4:0);
+			if (grfcontversion == 2 && datasize != (unsigned long)(sx * sy * bytes_per_pixel) + SpriteInfo::Size(grfcontversion)) {
 				printf("\nError: not enough data to perform regular decoding for sprite %d\n", spriteno);
 				exit(2);
 			}
@@ -572,7 +582,7 @@ static multitype strategy2(const U8* data, unsigned int datasize, unsigned int d
 #pragma warning(disable:4701)//chunk may be used uninitialized
 #endif
 
-static long realcompress(const U8 *in, long insize, U8 *out, long outsize, long *compsize)
+static long realcompress(const U8 *in, long insize, U8 *out, long outsize, long *compsize, int grfcontversion)
 {
 	long inpos = 0, outpos = 0;
 	U8 *lastcodepos = out;
@@ -606,8 +616,12 @@ static long realcompress(const U8 *in, long insize, U8 *out, long outsize, long
 			lastcodepos = &(out[outpos++]);
 			inpos += overlap;
 		} else {	//  no we didn't. Increase length of verbatim chunk
-			if (*lastcodepos == 0x80) {	// maximum length 128, encoded as 0
-				*lastcodepos = 0;
+			/* In container version 2, verbatim data of length 128 can be encoded using 0.
+			 * While various GRF format specs list this already for container version 1, it is not true.
+			 * See http://www.tt-forums.net/viewtopic.php?p=1051455#p1051455 for details. */
+			if (*lastcodepos == (grfcontversion >= 2 ? 0x80 : 0x7f)) {	// check for maximum verbatim length
+				if (*lastcodepos == 0x80)
+					*lastcodepos = 0;
 				lastcodepos = &(out[outpos++]);	// start new one
 				*lastcodepos = 0;
 			}
@@ -797,7 +811,7 @@ long encoderegular(U8 **compressed_data, const U8 *image, long imgsize, SpriteIn
 	while (1) {
 		inf.writetobuffer(compr, grfcontversion);
 		if (docompress)
-			result = realcompress(image, imgsize, compr+infobytes, compsize-infobytes, &realcompsize);
+			result = realcompress(image, imgsize, compr+infobytes, compsize-infobytes, &realcompsize, grfcontversion);
 		else
 			result = fakecompress(image, imgsize, compr+infobytes, compsize-infobytes, &realcompsize, grfcontversion);
 
